#!/usr/bin/env python3        

import argparse               
from collections import defaultdict  


def mean(lst):
    if lst:   
        return sum(lst) / len(lst)
    else:
        return None
    
    
def sd(lst):
    if not lst:
        return None       
    if len(lst) == 1:
        return 0
    else:
        m = mean(lst)
        return sum((item - m) ** 2 for item in lst) / (len(lst) - 1)


def load_file(fname):
    d = defaultdict(list)
    with open(fname, 'rt') as f:
        f.readline()                                     
        for line in f:
            if line.strip():                             
                name, value = line.strip().split('\t')   
                d[name].append(float(value))
    return d


def save_file(fname, data):
    """
    fname - file name
    data - dict, keys are compound names, values are tuples of average and sd activity values
    """
    with open(fname, "wt") as f:
        for k, v in output.items():
            f.write(k + "\t" + "\t".join(map(str, v)) + "\n")


if __name__ == '__main__':    # entry point of the script
    
    parser = argparse.ArgumentParser(description='Calculate average and standard deviation of compound activity.')
    parser.add_argument('-i', '--input', metavar='input.txt', required=True,
                        help='text file with header and two tab-separated column with compound name and '
                             'activity value.')
    parser.add_argument('-o', '--out', metavar='output.txt', required=False, default=None,
                        help='output text file in tab separated format with average and sd for each compound. '
                             'If the name will be omitted output will be in stdout.')

    args = vars(parser.parse_args())
    for k, v in args.items():
        if k == "input": input_fname = v
        if k == "out": output_fname = v
            
    d = load_file(input_fname)
    
    output = {k: (mean(v), sd(v)) for k, v in d.items()}
    
    if output_fname is None:                         # print to stdout (may also be done with sys.stdout.write)
        for k, v in output.items():
            print(k, *v, sep="\t")            # * unpacks iterable
    else:
        save_file(output_fname, output)
