#!/usr/bin/env python3

import os
import argparse
from multiprocessing import Process, Manager, cpu_count, Pool


def long_calc(n):
    i = 0
    while i < n * 500:
        i += 1
    return n * 2


def long_calc_queue(q, output_file, lock):
    
    while True:
        value = q.get()
        if value is None:
            break
        res = long_calc(value)
        lock.acquire()
        try:
            open(output_file, "at").write(str(res) + "\n")
        finally:
            lock.release()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Multiprocessing example. Read text file with numbers, calculate reasults and save them.')
    parser.add_argument('-i', '--input', metavar='input.txt', required=True,
                        help='input text file with single numbers on separate lines.')
    parser.add_argument('-o', '--out', metavar='output.txt', required=True,
                        help='output text file.')
    parser.add_argument('-c', '--ncpu', metavar='NCPU', required=False, default=None,
                        help='number of cores used for calculation. By default all but one will be used.')

    args = vars(parser.parse_args())
    for o, v in args.items():
        if o == "input": in_fname = v
        if o == "out": out_fname = v
        if o == "ncpu":
            ncpu = cpu_count() - 1 if v is None else max(min(int(v), cpu_count()), 1)

    if os.path.isfile(out_fname):
         os.remove(out_fname)

    manager = Manager()
    lock = manager.Lock()
    q = manager.Queue(10 * ncpu)

    pool = []
    for _ in range(ncpu):
        p = Process(target=long_calc_queue, args=(q, out_fname, lock))
        p.start()
        pool.append(p)

    with open(in_fname) as f:
        for line in f:
            v = line.strip()
            if v:
                q.put(int(v))

    for _ in range(ncpu):
        q.put(None)
    
    for p in pool:
        p.join()

