#!/usr/bin/env python3

import os
import argparse
from multiprocessing import Pool, cpu_count


def long_calc(n):
    if n.strip():
        n = int(n.strip())
        i = 0
        while i < n * 500:
            i += 1
        return n * 2


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Multiprocessing example. Read text file with numbers, calculate reasults and save them.')
    parser.add_argument('-i', '--input', metavar='input.txt', required=True,
                        help='input text file with single numbers on separate lines.')
    parser.add_argument('-o', '--out', metavar='output.txt', required=True,
                        help='output text file.')
    parser.add_argument('-c', '--ncpu', metavar='NCPU', required=False, default=None,
                        help='number of cores used for calculation. By default all but one will be used.')

    args = vars(parser.parse_args())
    for o, v in args.items():
        if o == "input": in_fname = v
        if o == "out": out_fname = v
        if o == "ncpu":
            ncpu = cpu_count() - 1 if v is None else max(min(int(v), cpu_count()), 1)

    if os.path.isfile(out_fname):
         os.remove(out_fname)
            
    p = Pool(ncpu)
    with open(out_fname, "wt") as f:
        for res in p.imap_unordered(long_calc, open(in_fname), chunksize=10):
            f.write(str(res) + "\n")
            
